var Histology = function()
{
	var histology = {};
	var debug = 0;

	var SCORES = ['ScoreInvasiveCancer', 'ScoreInsituCancer', 'ScoreLymphocytes', 'ScoreNormal', 'ScoreStroma', 'ScoreFat'];
	var FIELDS = ['score_invasive_cancer', 'score_insitu_cancer', 'score_lymphocytes', 'score_normal', 'score_stroma', 'score_fat'];

	var heGlass;
	var samples;
	var selectedSample;
	var disableModificationCheck = false;
	
	// Page initialization
	histology.initPage = function()
	{
		Events.addEventHandler('btnFindHeGlass', 'click', histology.searchHeGlass);
		Events.doOnEnter('findHeGlassName', histology.searchHeGlass);
		Buttons.addClickHandler('btnUncompletedHeGlass', histology.showUncompletedHeGlassMenu);
		Events.addEventHandler('select-uncompleted-heglass', 'click', histology.uncompletedGlassSelected);
		Events.addEventHandler('wizard', 'click', histology.hideUncompletedHeGlassMenu);
		Buttons.addClickHandler('btnSelectHeGlass', histology.selectHeGlass);
		Events.addEventHandler('btnSelectHeGlass', 'base-selected', histology.heGlassSelected);
		
		for (var i = 0; i < FIELDS.length; i++)
		{
			var f = FIELDS[i];
			Events.addEventHandler(f, 'keypress', Events.integerOnly);
			Events.addEventHandler(f, 'change', histology.checkScores);
		}
		
		Events.addEventHandler('good_stain', 'click', histology.goodStainOnClick);
		Events.addEventHandler('score_complete', 'click', histology.scoreCompleteOnClick);
		Events.addEventHandler('comments', 'change', histology.commentsOnChange);
		
		Events.addEventHandler('ihc_quality', 'change', histology.ihcQualityOnChange);
		
		// Image selection - drag/drop
		Events.addEventHandler('he-file-drop', 'drop', histology.onDropFile);
		Events.addEventHandler('he-file-drop', 'dragenter', histology.onDragEnter);
		Events.addEventHandler('he-file-drop', 'dragleave', histology.onDragLeave);
		Events.addEventHandler('he-file-drop', 'dragover', histology.onDragOver);

		// Image selection - regular selection via dialog
		Events.addEventHandler('no-he-image', 'click', histology.selectFile);
		Events.addEventHandler('selectFile', 'change', histology.fileSelected);
		
		// Image selection - copy/paste
		Events.addEventHandler(document, 'paste', histology.pasteImage);
		
		// Delete image
		Events.addEventHandler('he-image-delete', 'click', histology.deleteImage);

		// Zoom in popup window
		Events.addEventHandler('he-image', 'click', histology.imageOnClick);
		
		// Navigation
		Buttons.addClickHandler('gonext', histology.goNext);
		Buttons.addClickHandler('goregister', Wizard.goRegister);

		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', histology.submit);

		var url = '../Histology.servlet?ID='+App.getSessionId();
		url += '&cmd=GetHeGlassToScore';
		var glassName = Data.get('page-data', 'he-glass');
		if (glassName) url += '&name='+encodeURIComponent(glassName);

		Wizard.showLoadingAnimation(glassName ? 'Finding HE glass: '+Strings.encodeTags(glassName) : 'Loading next HE glass for scoring...');
		Wizard.asyncJsonRequest(url, histology.heGlassLoaded);
	}
	
	histology.heGlassLoaded = function(response)
	{
		var glassName = response.name;
		heGlass = response.heGlasses;

		Doc.show('step-1');

		if (heGlass.length > 0)
		{
			// Generate HE glass HTML
			var html = '';
			for (var glassNo = 0; glassNo < heGlass.length; glassNo++)
			{
				var glass = heGlass[glassNo];
				html += histology.createHeGlassSection(glass);
			}
			Doc.element('he-glass-container').innerHTML = html;
			Doc.show('he-glass-container');
			
			// Add handlers for viewing NDPI images
			var images = document.getElementsByClassName('ndpi-image');
			for (var i = 0; i < images.length; i++)
			{
				Events.addEventHandler(images[i], 'click', histology.previewNdpi);
			}
			var images = document.getElementsByClassName('open-ndpi');
			for (var i = 0; i < images.length; i++)
			{
				Events.addEventHandler(images[i], 'click', Files.viewFileOnClick);
			}
			
			// Fill form with know information
			var firstGoodSample;
			for (var glassNo = 0; glassNo < heGlass.length; glassNo++)
			{
				var glass = heGlass[glassNo];
				for (var sampleNo = 0; sampleNo < glass.samples.length; sampleNo++)
				{
					var sample = glass.samples[sampleNo];
					
					histology.saveOriginalProperties(sample);
					histology.restoreFromLocalStorage(sample);
					
					// Save ID of HE glass since we need that later
					sample.heGlassId = glass.id;
					sample.IHC = glass.IHC;
					
					// Save the first GoodStain sample so we can select it later
					if (sample.GoodStain)
					{
						if (firstGoodSample == null) firstGoodSample = sample;
					}
					
					
					// Generate text inside table cell
					var html = '<div>';
					html += '<div class="sample-name">'+Strings.encodeTags(sample.name.replace(/\.he\d+$/, ''))+'</div>';
					if (sample.IHC == null || sample.GoodStain)
					{
						// Calculate sum of scores
						var total = histology.calculateTotalScore(sample);
						html += '<div class="progress" id="'+sample.id+'.progress">' + total + '%</div>';
					}
					else
					{
						html += '<div class="progress" id="'+sample.id+'.progress">' + Strings.encodeTags(sample.IHC + ': ' + (sample.IHCQuality || '?')) + '</div>';
					}
					html += '<div class="thumbnail"></div>';
					html += '</div>';
					
					var td = Doc.element(sample.heGlassId+'-'+sample.well.location);
					td.innerHTML = html;
					td.sample = sample;
					
					histology.setSampleClassName(sample);
					if (sample.IHC == null || sample.GoodStain)
					{
						Events.addEventHandler(td, 'click', histology.sampleOnClick);
					}
					else
					{
						Events.addEventHandler(td, 'click', histology.ihcOnClick);
					}
				}
			}
			
			if (firstGoodSample != null)
			{
				Doc.show('score-container');
				histology.selectSample(firstGoodSample);
			}
			
			if (heGlass.length > 3)
			{
				// To make sure the image does not scroll out of view
				Doc.addClass('he-layout', 'many-he-glass');
			}
			
			Wizard.keepSessionAlive();
			Doc.show('goregister');
			window.addEventListener('beforeunload', histology.confirmLeaveWizard);
		}
		else
		{
			var msg;
			if (glassName)
			{
				msg = 'Could not find HE glass with name: ' + Strings.encodeTags(glassName);
			}
			else
			{
				msg = 'No HE glass available for scoring.';
				msg += ' Specify HE glass name above to re-score.';
			}
			Doc.element('he-glass-container').innerHTML = '<div class="messagecontainer note">'+msg+'</div>';
			Doc.show('he-glass-container');
			Doc.show('gonext');
		}
		
		
		var url = '../Histology.servlet?ID='+App.getSessionId();
		url += '&cmd=GetUncompletedHeGlass';
		Wizard.asyncJsonRequest(url, histology.uncompletedHeGlassLoaded);
	}
	
	/**
		Show a preview of either the label or the scanned area.
	*/
	histology.previewNdpi = function(event)
	{
		var img = event.currentTarget;
		// Find out the 'x' coordinage of the mouse click
		var bounds = img.getBoundingClientRect();
	    var left = bounds.left;
	    var height = img.clientHeight;
	    var x = event.pageX - left;

	    var fileId = Data.get(img, 'file-id');
		var url = '../../thumbnails.jar/Thumbnails.servlet?ID='+App.getSessionId()+'&file='+fileId+'&size=XXLARGE';
	    if (x < height)
	    {
	    	// The click was to the left, display the label (we know that it is at image index 7)
			url += '&index=7&crop=LABEL';
	    }
	    else
	    {
	    	// The click was to the right, display the scanned area (the smallest image is at image index 6)
	    	url += '&index=6';
	    }
		Doc.element('he-image-name').innerHTML = Data.get(img, 'glass-name');
		Doc.hide('file-error');
		Doc.addClass('he-image-container', 'has-image');
		Doc.removeClass('he-image-container', 'can-delete');
	    Data.set('he-image', 'file-id', fileId);
	    Doc.element('he-image').src = '../images/loading.gif';
	    Doc.element('he-image').src = url;
	}
	
	histology.uncompletedHeGlassLoaded = function(response)
	{
		var uncompleted = response.heGlasses;
		
		if (uncompleted.length > 0)
		{
			// Populate menu item
			Doc.element('btnUncompletedHeGlass.text').innerHTML = uncompleted.length + ' uncompleted';
			var html = '';	
			for (var glassNo = 0; glassNo < uncompleted.length; glassNo++)
			{
				var glass = uncompleted[glassNo];
			
				html += '<div class="menuitem enabled interactable" id="glass-'+glass.id+'" data-name="'+Strings.encodeTags(glass.name)+'">';
				html += Strings.encodeTags(glass.name)+' ('+glass.numUncompleteSamples +')</div>';
			}
			Doc.element('select-uncompleted-heglass-menu').innerHTML = html;
			Doc.show('btnNncompletedHeGlass');
		}
		else
		{
			Doc.hide('btnUncompletedHeGlass');
		}
	}

	histology.showUncompletedHeGlassMenu = function(event)
	{
		var pos = Doc.getElementPosition('btnUncompletedHeGlass');
		var x = pos.left;
		var y = pos.top+pos.height;

		var menu = Doc.element('select-uncompleted-heglass');
		menu.style.left = (x)+'px';
		menu.style.top = (y)+'px';
		menu.style.display = 'block';
		event.stopPropagation();
	}
	
	histology.hideUncompletedHeGlassMenu = function(event)
	{
		Doc.hide('select-uncompleted-heglass');
	}

	histology.uncompletedGlassSelected = function(event)
	{
		histology.findHeGlass(Data.get(event.target, 'name'));
	}

	
	histology.selectHeGlass = function(event)
	{
		if (!histology.confirmLeaveWizard()) return;
		
		var url = '&resetTemporary=1';
		url += '&tmpfilter:STRING:name='+encodeURIComponent('HE%');
		Dialogs.selectItem('BIOPLATE', event.currentTarget.id, 0, url);
	}
	
	histology.heGlassSelected = function(event)
	{
		disableModificationCheck = true;
		histology.findHeGlass(event.detail.name);
	}
	
	histology.searchHeGlass = function(event)
	{
		var frm = document.forms['reggie'];
		if (frm.findGlassName.value) histology.findHeGlass();
	}
	
	histology.findHeGlass = function(glassName, autoFind)
	{
		var frm = document.forms['reggie'];
		if (!glassName && autoFind != true)
		{
			glassName = frm.findGlassName.value;
			if (!glassName) return;
			
			if (glassName.match(/^\d+$/))
			{
				while (glassName.length < 5)
				{
					glassName = '0'+glassName;
				}
				glassName = 'HE'+glassName;
			}
		}
		var url =  'histology_score.jsp?ID='+App.getSessionId();
		if (glassName)
		{
			url += '&name='+encodeURIComponent(glassName);
		}
		
		location.replace(url);
	}

	histology.goNext = function()
	{
		histology.findHeGlass(null, true);
	}
	
	histology.createHeGlassSection = function(glass)
	{
		var html = '';
		html += '<div class="he-glass bg-filled-50">';
		html += '<div class="he-name bg-filled-100">';
		html += Strings.encodeTags(glass.name || '');
		html += '</div>';
		
		html += '<div class="he-info">';
		if (glass.IHC != null)
		{
			html += '<span class="ihc">' + Strings.encodeTags(glass.IHC) + '</span>; ';
		}
		html += '<span class="he-tray">' +  Strings.encodeTags(glass.tray || '') + '</span>; <span class="he-position">' + Strings.encodeTags(glass.position || '') + '</span>';
		html += '<div class="he-comment">' + Strings.encodeTags(glass.comments || '') + '</div>';
		html += '</div>';
		html += '<div class="he-thumb">';
		if (glass.ndpiImage)
		{
			html += '<img class="ndpi-image link" title="Preview NanoZoomer Digital Pathology Image"';
			html += ' src="../../thumbnails.jar/Thumbnails.servlet?ID='+App.getSessionId()+'&file='+glass.ndpiImage.id+'&size=LARGE&index=7"';
			html += ' data-file-id="'+glass.ndpiImage.id+'" data-glass-name="'+Strings.encodeTags(glass.name)+'"';
			html += '>';
			html += '<img class="open-ndpi link" data-file-id="'+glass.ndpiImage.id+'" title="Download and open NanoZoomer Digital Pathology Image" src="../images/ndp.png">';
		}
		else
		{
			html += 'No scanned image exists for this slide';
		}
		html += '</div>';
		html += '<table class="he-table">';
		html += '<tr>';
		html += '<td class="not-used" id="'+glass.id+'-A1"></td>';
		html += '<td class="not-used" id="'+glass.id+'-A2"></td>';
		html += '</tr>';
		html += '<tr>';
		html += '<td class="not-used" id="'+glass.id+'-B1"></td>';
		html += '<td class="not-used" id="'+glass.id+'-B2"></td>';
		html += '</tr>';
		html += '<tr>';
		html += '<td class="not-used" id="'+glass.id+'-C1"></td>';
		html += '<td class="not-used" id="'+glass.id+'-C2"></td>';
		html += '</tr>';
		html += '</table>';
		html += '</div>';

		return html;
	}

	/**
		Remember the original properties so that we know if
		a sample has been modified.
	*/
	histology.saveOriginalProperties = function(sample)
	{
		var o = {};
		for (var i = 0; i < SCORES.length; i++)
		{
			var p = SCORES[i];
			o[p] = sample[p];	
		}
		
		o.GoodStain = sample.GoodStain;
		o.ScoreComplete = sample.ScoreComplete;
		o.comments = sample.comments;
		o.IHCQuality = sample.IHCQuality;
		sample.o = o;
		sample.isModified = false;
	}
	
	/**
		Check if a sample has been modified.
	*/
	histology.isModified = function(sample, useCache)
	{
		if (!useCache)
		{
			var o = sample.o;
			sample.isModified = false;
			for (var p in o)
			{
				if (o[p] != sample[p])
				{
					sample.isModified = true;
					break;
				}
			}
		}
		return sample.isModified;
	}

	
	histology.calculateTotalScore = function(sample)
	{
		var total = 0;
		var numNull = 0;
		for (var i = 0; i < SCORES.length; i++)
		{
			var score = sample[SCORES[i]];
			total += histology.sumScore(score);
			if (score == null) numNull++;
		}
		// Set summed total, null only if all values are null
		sample.ScoreTotal = total == 0 && numNull == SCORES.length ? null : total;
		sample.numNullScores = numNull;
		return total;
	}
	
	histology.sumScore = function(score)
	{
		return score || 0;
	}

	histology.setSampleClassName = function(sample)
	{
		var className = 'used';
		if (sample.GoodStain)
		{
			className += ' good-stain';
		}
		else if (sample.IHC)
		{
			className += ' ihc';
			if (sample.IHCQuality)
			{
				className += ' ihc-done';
			}
		}
		else
		{
			if (histology.getGoodStainedSampleForLocation(sample.well.location) == null)
			{
				className += ' flag';
			}
		}
		
		if (sample.ScoreTotal == null)
		{
			className += ' score-none';
		}
		else if (sample.ScoreTotal != 100 || !sample.ScoreComplete)
		{
			className += ' score-incomplete';
		}
		else
		{
			className += ' score-complete';
		}
		
		if (sample.image)
		{
			className += ' has-image can-delete';
		}
		
		if (histology.isModified(sample))
		{
			className += ' modified';
		}
		
		if (selectedSample && selectedSample.id == sample.id)
		{
			className += ' selected';
		}
		
		var td = Doc.element(sample.heGlassId+'-'+sample.well.location);
		td.className = className;
	}

	histology.getGoodStainedSampleForLocation = function(location)
	{
		// Find sample on given location that has GoodStain annotation
		for (var glassNo = 0; glassNo < heGlass.length; glassNo++)
		{
			var glass = heGlass[glassNo];
			for (var sampleNo = 0; sampleNo < glass.samples.length; sampleNo++)
			{
				var sample = glass.samples[sampleNo];
				if (sample.well.location == location && sample.GoodStain)
				{
					return sample;
				}
			}
		}
		return null;
	}

	histology.sampleOnClick = function(event)
	{
		var td = event.currentTarget;
		histology.selectSample(td.sample);
	}
	
	histology.ihcOnClick = function(event)
	{
		var td = event.currentTarget;
		histology.selectSample(td.sample);
	}

	histology.selectSample = function(sample)
	{
		var frm = document.forms['reggie'];
		
		// Sample name
		Doc.element('sample.name').innerHTML = Strings.encodeTags(sample.name);
		
		if (sample.IHC)
		{
			Doc.hide('heSample');
			Doc.show('ihcSample');
			
			Forms.selectListOption(frm.ihc_quality, sample.IHCQuality || '');
			Doc.element('ihc_name').innerHTML = Strings.encodeTags(sample.IHC);
		}
		else
		{
			Doc.show('heSample');
			Doc.hide('ihcSample');
			
			// Scores
			for (var i = 0; i < SCORES.length; i++)
			{
				frm[FIELDS[i]].value = sample[SCORES[i]];
			}
			
			// Total score + score complete
			var total = histology.calculateTotalScore(sample);
			Doc.element('score_total').innerHTML = total;
			if (total == 100)
			{
				if (sample.numNullScores == 0)
				{
					frm.score_complete.disabled = false;
					frm.score_complete.checked = sample.ScoreComplete;
					Wizard.setInputStatus('score_total', 'valid');
				}
				else
				{
					frm.score_complete.disabled = true;
					frm.score_complete.checked = false;
					Wizard.setInputStatus('score_total', 'warning', 'Enter 0 in empty fields to complete scoring');
				}
			}
			else
			{
				frm.score_complete.disabled = true;
				frm.score_complete.checked = false;
				Wizard.setInputStatus('score_total', 'warning', 'Not 100%');
			}
			
			// Good stain
			if (sample.GoodStain)
			{
				frm.good_stain.checked = true;
			}
			else
			{
				frm.good_stain.checked = false;
			}
			frm.score_invasive_cancer.focus();
		}

		// Image
		Doc.element('he-image-name').innerHTML = Strings.encodeTags(sample.name);
		Doc.hide('file-error');
		if (sample.image)
		{
			Doc.element('he-image').src = App.getRoot() + 'filemanager/files/view/-'+App.getSessionId()+'-' + sample.image;
			Data.set('he-image', 'file-id', 0);
			Doc.addClass('he-image-container', 'has-image');
			Doc.addClass('he-image-container', 'can-delete');
		}
		else
		{
			Doc.removeClass('he-image-container', 'has-image');
		}
		
		// Comments
		frm.comments.value = sample.comments || '';
		
		// Class name of selected table cell
		if (selectedSample != null)
		{
			Doc.removeClass(selectedSample.heGlassId+'-'+selectedSample.well.location, 'selected');
		}
		
		selectedSample = sample;
		histology.setSampleClassName(sample);
		
	}
	
	histology.checkScores = function()
	{
		var frm = document.forms['reggie'];

		for (var i = 0; i < SCORES.length; i++)
		{
			selectedSample[SCORES[i]] = histology.toScore(frm[FIELDS[i]].value);
		}
		
		var oldTotal = selectedSample.ScoreTotal;
		var oldNumNull = selectedSample.numNullScores;
		
		var total = histology.calculateTotalScore(selectedSample);
		
		Doc.element('score_total').innerHTML = total;
		Doc.element(selectedSample.id+'.progress').innerHTML = total + '%';
		
		if (total == 100)
		{
			if (selectedSample.numNullScores == 0)
			{
				// All scores have been entered
				frm.score_complete.disabled = false;
				if (oldTotal != total || oldNumNull != selectedSample.numNullScores)
				{
					// Only check the 'Score complete' if value has changed
					frm.score_complete.checked = true;
					selectedSample.ScoreComplete = true;
				}
				Wizard.setInputStatus('score_total', 'valid');
			}
			else
			{
				frm.score_complete.disabled = true;
				frm.score_complete.checked = false;
				selectedSample.ScoreComplete = false;
				Wizard.setInputStatus('score_total', 'warning', 'Enter 0 in empty fields to complete scoring');
			}
		}
		else
		{
			frm.score_complete.disabled = true;
			frm.score_complete.checked = false;

			Wizard.setInputStatus('score_total', 'warning', 'Not 100%');
			selectedSample.ScoreComplete = selectedSample.ScoreTotal == null ? null : false;
		}

		histology.setSampleClassName(selectedSample);
		histology.saveToLocalStorage(selectedSample);
	}

	histology.toScore = function(value)
	{
		var score = parseInt(value);
		return isNaN(score) ? null : score;
	}

	histology.goodStainOnClick = function()
	{
		var frm = document.forms['reggie'];
		
		var goodSample = histology.getGoodStainedSampleForLocation(selectedSample.well.location);
		if (goodSample != null && goodSample != selectedSample)
		{
			// Reset the current GoodStain sample
			goodSample.GoodStain = null;
			histology.saveToLocalStorage(goodSample);
		}
		
		selectedSample.GoodStain = frm.good_stain.checked ? true : null;
		histology.saveToLocalStorage(selectedSample);
		
		// Repaint samples on current location
		for (var glassNo = 0; glassNo < heGlass.length; glassNo++)
		{
			var glass = heGlass[glassNo];
			
			for (var sampleNo = 0; sampleNo < glass.samples.length; sampleNo++)
			{
				var sample = glass.samples[sampleNo];
				if (sample.well.location == selectedSample.well.location)
				{
					histology.setSampleClassName(sample);
				}
			}
		}
	}
	
	histology.scoreCompleteOnClick = function()
	{
		var frm = document.forms['reggie'];
		selectedSample.ScoreComplete = frm.score_complete.checked;
		histology.setSampleClassName(selectedSample);
		histology.saveToLocalStorage(selectedSample);
	}

	histology.commentsOnChange = function()
	{
		var frm = document.forms['reggie'];
		selectedSample.comments = frm.comments.value;
		histology.setSampleClassName(selectedSample);
		histology.saveToLocalStorage(selectedSample);
	}
	
	histology.ihcQualityOnChange = function()
	{
		var frm = document.forms['reggie'];
		selectedSample.IHCQuality = frm.ihc_quality.value || null;
		Doc.element(selectedSample.id+'.progress').innerHTML = Strings.encodeTags(selectedSample.IHC + ': ' + (selectedSample.IHCQuality || '?'));
		histology.setSampleClassName(selectedSample);
		histology.saveToLocalStorage(selectedSample);
	}

	// Open a popup with the full image
	histology.imageOnClick = function()
	{
		var img = Doc.element('he-image');
		var fileId = Data.int(img, 'file-id', 0);
		if (fileId && fileId > 0)
		{
			Files.viewFile(fileId);
		}
		else if (img.src)
		{
			var options = 'toolbar=no,location=no,directories=no,status=no,menubar=no,scrollbars=yes,resizable=yes';
			if (img.naturalHeight && img.naturalWidth)
			{
				options += ',width='+(img.naturalWidth+20)+',height='+(img.naturalHeight+20);
			}
			window.open(img.fullSrc || img.src, 'Image'+selectedSample.name, options);
		}
	}
	
	histology.selectFile = function()
	{
		Doc.element('selectFile').click();
	}
	
	histology.fileSelected = function()
	{
		var files = Doc.element('selectFile').files;
		if (files.length > 0)
		{
			// The file to upload
			histology.uploadFile(files[0]);
		}
	}
	
	// Visual feedback when drop target zone is entered
	histology.onDragEnter = function(event)
	{
		Doc.addClass('he-image-container', 'dragging');
		Doc.hide('file-error');
	}
	
	// Visual feedback when leaving drop target zone
	histology.onDragLeave = function(event)
	{
		Doc.removeClass('he-image-container', 'dragging');
	}

	// Need to prevent default event to stop browser from loading the dropped file
	histology.onDragOver = function(event)
	{
		event.preventDefault();
	}

	// A file has been dropped
	histology.onDropFile = function(event)
	{
		event.preventDefault();
		Doc.removeClass('he-image-container', 'dragging');
		var files = event.dataTransfer.files;
		if (files.length > 0)
		{
			// The file to upload
			histology.uploadFile(files[0]);
		}
	}
	
	// Check if paste event contains an image
	histology.pasteImage = function(event)
	{
		var clipboardItems = event.clipboardData.items;
		if (!clipboardItems) return;
		
		for (var i = 0; i < clipboardItems.length; i++)
		{
			var item = clipboardItems[i];
			if (item.type == 'image/png' || item.type == 'image/jpg' || item.type == 'image/tiff')
			{
				var file = item.getAsFile();
				if (file) histology.uploadFile(file);
				event.preventDefault();
				return;
			}
		}
	}
	
	histology.uploadFile = function(file)
	{
		if (selectedSample.image)
		{
			if (!confirm('Replace the current image with \'' + file.name + '\'?')) return;
		}
		
		if (file.type != 'image/png' && file.type != 'image/jpeg' && file.type != 'image/tiff')
		{
			histology.showFileUploadError('The file must be a TIF, JPG or PNG file: ' + file.type);
			return;
		}
		
		Doc.removeClass('he-image-container', 'has-image');
		Doc.addClass('he-image-container', 'uploading');

		var url = '../Histology.servlet?ID='+App.getSessionId();
		url += '&cmd=UploadImage';
		url += '&sampleId='+selectedSample.id;
		url += '&filename='+encodeURIComponent(file.name);

		var upload = Ajax.getXmlHttpRequest();
		upload.open('POST', url, true);
		Ajax.setReadyStateHandler(upload, histology.fileLoaded, histology.fileLoaded);
		upload.send(file);	
	}
	
	histology.showFileUploadError = function(msg)
	{
		var errorDiv = Doc.element('file-error');
		errorDiv.innerHTML = msg;
		Doc.show(errorDiv);
	}
	
	// Display the loaded file in the preview section
	histology.fileLoaded = function(request)
	{
		Doc.removeClass('he-image-container', 'uploading');
		response = JSON.parse(request.responseText);
		if (response.status != 'ok')
		{
			histology.showFileUploadError(response.message || response.stacktrace);
			return;
		}

		selectedSample.image = response.image+'?'+(new Date().getTime()); // To avoid problems with caching the old image
		Doc.element('he-image').src = App.getRoot() + 'filemanager/files/view/-'+App.getSessionId()+'-' + selectedSample.image;
		Data.set('he-image', 'file-id', 0);
		Doc.addClass('he-image-container', 'has-image');
		Doc.addClass('he-image-container', 'can-delete');
		Doc.hide('file-error');
		histology.setSampleClassName(selectedSample);
	}
	
	histology.deleteImage = function()
	{
		if (!confirm('Delete this image?')) return;
		
		var url = '../Histology.servlet?ID='+App.getSessionId();
		url += '&cmd=DeleteImage';
		url += '&sampleId='+selectedSample.id;
		
		var req = Ajax.getXmlHttpRequest();
		req.open('POST', url, true);
		Ajax.setReadyStateHandler(req, histology.fileDeleted, histology.fileDeleted);
		req.send();			
	}
	
	histology.fileDeleted = function(request)
	{
		response = JSON.parse(request.responseText);
		if (response.status != 'ok')
		{
			histology.showFileUploadError(response.message || response.stacktrace);
			return;
		}

		selectedSample.image = null;
		Doc.removeClass('he-image-container', 'has-image');
		Doc.hide('file-error');
		histology.setSampleClassName(selectedSample);
	}
	
	histology.submit = function()
	{
		samples = [];
		var glassInfo = [];
		
		for (var glassNo = 0; glassNo < heGlass.length; glassNo++)
		{
			var glass = heGlass[glassNo];
			var scoreCompleteForGlass = true;
			for (var sampleNo = 0; sampleNo < glass.samples.length; sampleNo++)
			{
				var sample = glass.samples[sampleNo];
				if (histology.isModified(sample))
				{
					samples[samples.length] = sample;
				}
				
				if (glass.IHC)
				{
					if (!sample.IHCQuality) scoreCompleteForGlass = false;
				}
				else
				{
					if (sample.GoodStain && sample.ScoreComplete != true || sample.ScoreComplete == false)
					{
						scoreCompleteForGlass = false;
					}
				}
			}
			
			var tmp = {};
			tmp.id = glass.id;
			tmp.ScoreComplete = scoreCompleteForGlass;
			glassInfo[glassInfo.length] = tmp;
		}
		
		if (samples.length == 0)
		{
			if (!confirm('Nothing has changed! Continue saving?'))
			{
				return;
			}
		}
		
		Doc.hide('score-container');
		Doc.addClass('he-glass-container', 'disabled');
			
		var submitInfo = {};
		submitInfo.samples = samples;
		submitInfo.heGlass = glassInfo;
		
		var url = '../Histology.servlet?ID='+App.getSessionId();
		url += '&cmd=SaveHeScore';
		Wizard.showLoadingAnimation('Saving...');
		Wizard.asyncJsonRequest(url, histology.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	histology.submissionResults = function(response)
	{
		// Reset changed properties and remove saved info
		for (var sampleNo = 0; sampleNo < samples.length; sampleNo++)
		{
			var sample = samples[sampleNo];
			histology.saveOriginalProperties(sample);
			histology.setSampleClassName(sample);
			histology.removeFromLocalStorage(sample);
		}
		
		disableModificationCheck = true;
		Wizard.showFinalMessage(response.messages);
		Doc.show('gonext');
	}

	
	/**
		Confirm if leaving the page is ok. If no event is passed this method
		is called manually and need to display dialog, otherwise the browser
		is taking care of this automatically.
	*/
	histology.confirmLeaveWizard = function(event)
	{
		if (disableModificationCheck) return;
		
		var numModified = 0;
		for (var glassNo = 0; glassNo < heGlass.length; glassNo++)
		{
			var glass = heGlass[glassNo];
			for (var sampleNo = 0; sampleNo < glass.samples.length; sampleNo++)
			{
				var sample = glass.samples[sampleNo];
				if (histology.isModified(sample))
				{
					numModified++;
				}
			}
		}
		
		if (event) 
		{
			if (numModified > 0) event.preventDefault();
		}
		else
		{
			if (numModified > 0)
			{
				return confirm('There are unsaved modifications. Continue?');
			}
			return true;
		}
	}

	
	/**
		Save score information to the local storage.
	*/
	histology.saveToLocalStorage = function(sample)
	{
		if (histology.isModified(sample, true))
		{
			var storeKey = histology.getStoreKey(sample);
			App.setLocal(storeKey, JSON.stringify(sample));
			if (debug)
			{
				App.debug('saveToLocalStorage: ' + sample.name + '=' + App.getLocal(storeKey));
			}
		}
	}
	
	histology.removeFromLocalStorage = function(sample)
	{
		if (debug) App.debug('removeFromLocalStorage:' + sample.name);
		App.removeLocal(histology.getStoreKey(sample));
	}
	
	histology.restoreFromLocalStorage = function(sample)
	{
		try
		{
			var storeKey = histology.getStoreKey(sample);
			var stored = App.getLocal(storeKey);
			if (debug) App.debug(stored);
			if (!stored) return;
			
			var tmp = JSON.parse(stored);
			if (!tmp) return;

			// Check that all original scores are the same, or someone has modified
			// this sample from another computer
			for (var p in sample.o)
			{
				if (sample.o[p] != tmp.o[p])
				{
					return;
				}
			}
			
			// Ok, copy stored info to the current sample
			for (var i = 0; i < SCORES.length; i++)
			{
				sample[SCORES[i]] = tmp[SCORES[i]];
			}
			sample.ScoreComplete = tmp.ScoreComplete;
			sample.GoodStain = tmp.GoodStain;
			sample.comments = tmp.comments;
			sample.IHCQuality = tmp.IHCQuality;
		}
		catch (e)
		{
			App.debug('Could not load stored information for sample ' + sample.name + ': ' + e);
		}
	}
	
	histology.getStoreKey = function(sample)
	{
		return 'reggie.histology.' + sample.id + '.scoreInfo';
	}

	return histology;
}();

Doc.onLoad(Histology.initPage);

